#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file was created using example of the python library impacket.
# Impacket project: https://github.com/CoreSecurity/impacket
#
# win_conn retunt a list of wmi objects of the query executed.
#


from impacket.dcerpc.v5.dcom import wmi
from impacket.dcerpc.v5.dtypes import NULL
from impacket.dcerpc.v5.dcomrt import DCOMConnection
from impacket.dcerpc.v5.dcom.wmi import DCERPCException


def wmi_conn(address, username, password, domain, namespace, query):
    """
    Create a connection with msrpc to get objects from wmi classes
    Will return a list of this objects to be read posterioly
    Others arguments can be used with the examples on the impacket library
    To connect will be used only user, password, domain and address from user
    """
    _dcom = DCOMConnection(address, username, password, domain, oxidResolver=True, doKerberos=False, kdcHost=address)
    _interface = _dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login, wmi.IID_IWbemLevel1Login)
    _web_level1_login = wmi.IWbemLevel1Login(_interface)
    _iwb_nt_login = _web_level1_login.NTLMLogin(namespace, NULL, NULL)
    _web_level1_login.RemRelease()
    _iwb_class_object = _iwb_nt_login.ExecQuery(query)
    _list_wmi_data = []
    try:
        while True:
            _list_wmi_data.append(_iwb_class_object.Next(0xffffffff, 1)[0])
    except DCERPCException:
        pass
    _iwb_nt_login.RemRelease()
    _dcom.disconnect()

    return _list_wmi_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The class wmi_query returns a defaultdict with objects of specific class
# 

import wmi_conn
from collections import defaultdict


class wmi_query(object):

    def __init__(self, opts):
        self.user = opts['user']
        self.password = opts['password']
        self.host = opts['address']
        self.domain = opts['domain']
        self.namespace = opts['namespace']
        self.query = opts['query']
        self.delimiter = "|"
        self.data_dict = defaultdict(lambda: False)
        self.dict_name = ""

    def get_wmi_data(self):
        """
        Get a list of wmi objects from wmi_conn and process the list into a default dict.
        """
        _wmi_data = wmi_conn.wmi_conn(self.host, self.user, self.password, self.domain, self.namespace, self.query)
        _cont = 0
        self.dict_name = _wmi_data[0].getClassName()
        self.data_dict[self.dict_name] = defaultdict()
        for data in _wmi_data:
            self.data_dict[self.dict_name][_cont] = defaultdict(lambda: False)
            data_properties = data.getProperties()
            for item in data_properties:
                self.data_dict[self.dict_name][_cont][item] = data_properties[item]['value']
            _cont = _cont + 1

    def get_item(self, item_name, value_name):
        """
        Return an entire object based in the value of the specific key passed with arguments.
        Ex: object.get_item('OS_Name', 'Windows 10')
        Will return each object with OS_Name like Windows 10.
        """
        return [self.data_dict[self.dict_name][x] for x, y in enumerate(self.data_dict[self.dict_name])
                if value_name in self.data_dict[self.dict_name][x][item_name]]

    def get_items(self, item_name):
        """
        Return only item passed with arguments for each object founded by the query.
        Eg: object.get_items('OS_Name')
        """
        return set([self.data_dict[self.dict_name][x][item_name] for x, y in enumerate(self.data_dict[self.dict_name])])

    def get_item_keys(self):
        """
        Return all keys of the object founded by the query.
        """
        return [x for x in self.data_dict[self.dict_name][0]]

    def name(self):
        """
        Return the class name research by the query.
        """
        return self.dict_name

    def run(self):
        self.get_wmi_data()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Return arguments for the script wmi_query.
#


import argparse


def set_parser():
    parser = argparse.ArgumentParser(add_help=True, description="Get wmi classes objects. Use -h to get help!")

    parser.add_argument(
        "-u", dest="user", action="store",
        help="User to connect on the machine."
        )
    parser.add_argument(
        "-p", dest="password", action="store",
        help="Password of the user to connect on the machine."
        )
    parser.add_argument(
        "-a", dest="address", action="store",
        help="IP of the Host to get wmi queries. Eg: 192.168.1.1"
        )
    parser.add_argument(
        "-d", dest="domain", action="store", default="WORKGROUP",
        help="Domain to check. Default: WORKGROUP."
        )
    parser.add_argument(
        "-n", dest="namespace", action="store", default="ROOT/cimv2",
        help="Namespace for classes. Default: ROOT/cimv2"
        )
    parser.add_argument(
        "-q", dest="query", action="store",
        help="Query to consult wmi databases. Eg: 'SELECT * FROM Win32_process'"
        )

    return parser



from setuptools import setup, find_packages

setup(
    name="wmi-query",
    version="0.1.3",
    license="BSD2CLAUSE",
    packages=find_packages(),
    data_files=[('', ['LICENSE'])],
    package_data={'': ['LICENSE']},
    keywords="wmi impacket",
    url="https://github.com/kanazux/wmi-query",
    author='Silvio AS a.k.a kanazuchi',
    author_email='alvolivre@live.com',
    scripts=['scripts/wmi-query'],
    description="A simple way to convert wmi data from wmi class objects into defaultdict.",
    install_requires=['impacket'],
)
